@@ -84,7 +84,6 @@ sub new
 	system_log_events_hash => undef
     };
     my (%options) = @_;
-    my ($rootDir,$outMethod,$systemLogEvents);
     while (my ($key,$value) = each(%options)) {
 	next if $key =~ /out_method|system_log_events/;
 	if (exists($fields{$key})) {
@@ -173,7 +172,9 @@ sub _initialize
     # Preloads
     #
     if ($self->preloads) {
+	die "list reference expected for preloads parameter" unless ref($self->preloads) eq 'ARRAY';
 	foreach my $pattern (@{$self->preloads}) {
+	    die "preloads pattern must be an absolute path" unless substr($pattern,0,1) eq '/';
 	    my @paths = $self->resolver->glob_path($pattern,$self);
 	    foreach (@paths) { $self->load($_) }
 	}
@@ -228,7 +229,7 @@ sub check_reload_file {
 	my @lines = split("\n",$block);
 	foreach my $comp_path (@lines) {
 	    if (exists($self->{code_cache}->{$comp_path})) {
-		$self->{code_cache_current_size} -= $self->{code_cache}->{$comp_path}->{size};
+		$self->{code_cache_current_size} -= $self->{code_cache}->{$comp_path}->{comp}->object_size;
 		delete($self->{code_cache}->{$comp_path});
 	    }
 	}
@@ -269,18 +270,12 @@ sub load {
     my $code_cache = $self->{code_cache};
     my $resolver = $self->{resolver};
 
-    #
-    # Use resolver to look up component and get fully-qualified path.
-    # Return undef if component not found.
-    #
-    my (@lookup_info) = $resolver->lookup_path($path,$self);
-    my $fq_path = $lookup_info[0] or return undef;
-
     #
     # If using reload file, assume that we are using object files and
     # have a cached subroutine or object file.
     #
     if ($self->{use_reload_file}) {
+	my $fq_path = $path;   # note - this will foil multiple component roots
 	return $code_cache->{$fq_path}->{comp} if exists($code_cache->{$fq_path});
 
 	$objfile = $self->object_dir . $fq_path;
@@ -295,6 +290,13 @@ sub load {
 	return $comp;
     }
 
+    #
+    # Use resolver to look up component and get fully-qualified path.
+    # Return undef if component not found.
+    #
+    my (@lookup_info) = $resolver->lookup_path($path,$self);
+    my $fq_path = $lookup_info[0] or return undef;
+
     #
     # Get last modified time of source.
     #
